// Copyright (c) 2020 Famedly GmbH
// SPDX-License-Identifier: AGPL-3.0-or-later

import 'encrypted_file.dart';
import 'src/subtle.dart';

import 'dart:convert';
import 'dart:math';
import 'dart:typed_data';

Future<EncryptedFile> encryptFile(Uint8List input) async {
  final rng = Random.secure();
  final key = await generateKey(AesKeyGenParams(name: 'AES-CTR', length: 256),
      true, ['encrypt', 'decrypt']);
  final iv = Uint8List(16);
  iv.setAll(0, Iterable.generate(8, (i) => rng.nextInt(256)));
  final data = (await encrypt(
          AesCtrParams(name: 'AES-CTR', counter: iv, length: 64), key, input))
      .asUint8List();
  final hash = await digest('SHA-256', data);
  final Jwk jwk = await exportKey('jwk', key);
  return EncryptedFile(
    data: data,
    k: jwk.k,
    iv: base64.encode(iv).replaceAll('=', ''),
    sha256: base64.encode(hash.asUint8List()).replaceAll('=', ''),
  );
}

Future<Uint8List> decryptFile(EncryptedFile input) async {
  final computed_hash = await digest('SHA-256', input.data);
  if (base64.encode(computed_hash.asUint8List()) !=
      base64.normalize(input.sha256)) {
    return null;
  }

  final key = await importKey(
      'jwk',
      Jwk(
          alg: 'A256CTR',
          ext: true,
          k: input.k,
          key_ops: ['encrypt', 'decrypt'],
          kty: 'oct'),
      'AES-CTR',
      true,
      ['encrypt', 'decrypt']);
  final iv = Uint8List(16);
  iv.setAll(0, base64.decode(base64.normalize(input.iv)));
  return (await encrypt(AesCtrParams(name: 'AES-CTR', counter: iv, length: 64),
          key, input.data))
      .asUint8List();
}
