// Copyright (c) 2020 Famedly GmbH
// SPDX-License-Identifier: AGPL-3.0-or-later

import 'encrypted_file.dart';

import 'package:pointycastle/pointycastle.dart';
import 'package:olm/olm.dart' as olm;

import 'dart:convert';
import 'dart:typed_data';
import 'dart:math';

String _sha256(Uint8List input) {
  final utility = olm.Utility();
  final result = utility.sha256_bytes(input);
  utility.free();
  return result;
}

Future<EncryptedFile> encryptFile(Uint8List input) async {
  final rng = Random.secure();
  final key = Uint8List(32);
  key.setAll(0, Iterable.generate(key.length, (i) => rng.nextInt(256)));
  final iv = Uint8List(16);
  iv.setAll(0, Iterable.generate(8, (i) => rng.nextInt(256)));

  final sc = StreamCipher('AES/CTR');
  sc.init(true, ParametersWithIV(KeyParameter(key), iv));

  final data = sc.process(input);
  return EncryptedFile(
    data: data,
    k: base64Url.encode(key).replaceAll('=', ''),
    iv: base64.encode(iv).replaceAll('=', ''),
    sha256: _sha256(data),
  );
}

Future<Uint8List> decryptFile(EncryptedFile input) async {
  if (base64.normalize(_sha256(input.data)) != base64.normalize(input.sha256)) {
    return null;
  }

  final key = base64.decode(base64.normalize(input.k));
  final iv = base64.decode(base64.normalize(input.iv));
  final sc = StreamCipher('AES/CTR');
  sc.init(false, ParametersWithIV(KeyParameter(key), iv));

  return sc.process(input.data);
}
