// Copyright (c) 2020 Famedly GmbH
// SPDX-License-Identifier: AGPL-3.0-or-later

@JS()
library subtle;

import 'package:js/js.dart';
import 'dart:async';
import 'dart:js_util';
import 'dart:typed_data';

@JS()
class CryptoKey {}

@JS()
@anonymous
class AesCtrParams {
  external factory AesCtrParams({String name, Uint8List counter, int length});
  String name;
  Uint8List counter;
  int length;
}

@JS()
@anonymous
class AesKeyGenParams {
  external factory AesKeyGenParams({String name, int length});
  String name;
  int length;
}

@JS()
@anonymous
class Jwk {
  external factory Jwk(
      {String alg, bool ext, String k, List<String> key_ops, String kty});
  String alg;
  bool ext;
  String k;
  List<String> key_ops;
  String kty;
}

// {"alg":"A256CTR","ext":true,"k":"krsEkcfVRby8aorK1nWKmRdTZOMDW7BYcCWMhr7zFCY","key_ops":["encrypt","decrypt"],"kty":"oct"}

@JS('crypto.subtle.generateKey')
external dynamic _generateKey(
    dynamic algorithm, bool extractable, List<String> keyUsages);

Future<CryptoKey> generateKey(
    dynamic algorithm, bool extractable, List<String> keyUsages) {
  return promiseToFuture(_generateKey(algorithm, extractable, keyUsages));
}

@JS('crypto.subtle.importKey')
external dynamic _importKey(String format, dynamic keyData, dynamic algorithm,
    bool extractable, List<String> keyUsages);

Future<CryptoKey> importKey(String format, dynamic keyData, dynamic algorithm,
    bool extractable, List<String> keyUsages) {
  return promiseToFuture(
      _importKey(format, keyData, algorithm, extractable, keyUsages));
}

@JS('crypto.subtle.exportKey')
external dynamic _exportKey(String algorithm, CryptoKey key);

Future<dynamic> exportKey(String algorithm, CryptoKey key) {
  return promiseToFuture(_exportKey(algorithm, key));
}

@JS('crypto.subtle.encrypt')
external dynamic _encrypt(dynamic algorithm, CryptoKey key, Uint8List data);

Future<ByteBuffer> encrypt(dynamic algorithm, CryptoKey key, Uint8List data) {
  return promiseToFuture(_encrypt(algorithm, key, data));
}

@JS('crypto.subtle.decrypt')
external dynamic _decrypt(dynamic algorithm, CryptoKey key, Uint8List data);

Future<ByteBuffer> decrypt(dynamic algorithm, CryptoKey key, Uint8List data) {
  return promiseToFuture(_decrypt(algorithm, key, data));
}

@JS('crypto.subtle.digest')
external dynamic _digest(String algorithm, Uint8List data);

Future<ByteBuffer> digest(String algorithm, Uint8List data) {
  return promiseToFuture(_digest(algorithm, data));
}
